@@ -63,8 +63,7 @@ pprCs :: [Cmm] -> SDoc
 pprCs cmms = pprCode CStyle (vcat $ map (\c -> split_marker $$ pprC c) cmms)
 
 writeCs :: Handle -> [Cmm] -> IO ()
-writeCs handle cmms = printForUser handle alwaysQualify (pprCs cmms)
- 			-- ToDo: should be printForC
+writeCs handle cmms = printForC handle (pprCs cmms)
 
 split_marker
   | opt_EnsureSplittableC = ptext SLIT("__STG_SPLIT_MARKER")
@@ -182,6 +181,10 @@ pprStmt stmt = case stmt of
 	-> ptext SLIT("ASSIGN_Word64") <> 
 		parens (mkP_ <> pprExpr1 dest <> comma <> pprExpr src) <> semi
 
+	| rep == F64 && wordRep /= I64
+	-> ptext SLIT("ASSIGN_DBL") <> 
+		parens (mkP_ <> pprExpr1 dest <> comma <> pprExpr src) <> semi
+
  	| otherwise
 	-> hsep [ pprExpr (CmmLoad dest rep), equals, pprExpr src <> semi ]
 	where
@@ -298,6 +301,9 @@ pprExpr e = case e of
     CmmLoad e I64 | wordRep /= I64
 	-> ptext SLIT("PK_Word64") <> parens (mkP_ <> pprExpr1 e)
 
+    CmmLoad e F64 | wordRep /= I64
+	-> ptext SLIT("PK_DBL") <> parens (mkP_ <> pprExpr1 e)
+
     CmmLoad (CmmReg r) rep 
 	| isPtrReg r && rep == wordRep
 	-> char '*' <> pprAsPtrReg r
@@ -364,8 +370,8 @@ pprLit :: CmmLit -> SDoc
 pprLit lit = case lit of
     CmmInt i rep      -> pprHexVal i rep
     CmmFloat f rep     -> parens (machRepCType rep) <> (rational f)
-    CmmLabel clbl      -> mkW_ <> pprCLabel clbl
-    CmmLabelOff clbl i -> mkW_ <> pprCLabel clbl <> char '+' <> int i
+    CmmLabel clbl      -> mkW_ <> pprCLabelAddr clbl
+    CmmLabelOff clbl i -> mkW_ <> pprCLabelAddr clbl <> char '+' <> int i
     CmmLabelDiffOff clbl1 clbl2 i
         -- WARNING:
         -- * the lit must occur in the info table clbl2
@@ -373,7 +379,9 @@ pprLit lit = case lit of
         -- The Mangler is expected to convert any reference to an SRT,
         -- a slow entry point or a large bitmap
         -- from an info table to an offset.
-        -> mkW_ <> pprCLabel clbl1 <> char '+' <> int i
+        -> mkW_ <> pprCLabelAddr clbl1 <> char '+' <> int i
+
+pprCLabelAddr lbl = char '&' <> pprCLabel lbl
 
 pprLit1 :: CmmLit -> SDoc
 pprLit1 lit@(CmmLabelOff _ _) = parens (pprLit lit)
@@ -638,6 +646,7 @@ isStrangeTypeReg (CmmGlobal g) 	= isStrangeTypeGlobal g
 isStrangeTypeGlobal :: GlobalReg -> Bool
 isStrangeTypeGlobal CurrentTSO		= True
 isStrangeTypeGlobal CurrentNursery 	= True
+isStrangeTypeGlobal BaseReg	 	= True
 isStrangeTypeGlobal r 			= isPtrGlobalReg r
 
 strangeRegType :: CmmReg -> Maybe SDoc
@@ -696,7 +705,7 @@ pprCall ppr_fn cconv results args vols
   where 
      ppr_results []     = empty
      ppr_results [(one,hint)] 
-	 = pprExpr (CmmReg one) <> ptext SLIT(" = ")
+	 = pprReg one <> ptext SLIT(" = ")
 		 <> pprUnHint hint (cmmRegRep one)
      ppr_results _other = panic "pprCall: multiple results"
 
@@ -980,11 +989,16 @@ pprHexVal w rep
   | otherwise = ptext SLIT("0x") <> go w <> repsuffix rep
   where
   	-- type suffix for literals:
-	-- on 32-bit platforms, add "LL" to 64-bit literals
-      repsuffix I64 | wORD_SIZE == 4 = ptext SLIT("LL")
+	-- Integer literals are unsigned in Cmm/C.  We explicitly cast to
+	-- signed values for doing signed operations, but at all other
+	-- times values are unsigned.  This also helps eliminate occasional
+	-- warnings about integer overflow from gcc.
+
+	-- on 32-bit platforms, add "ULL" to 64-bit literals
+      repsuffix I64 | wORD_SIZE == 4 = ptext SLIT("ULL")
       	-- on 64-bit platforms with 32-bit int, add "L" to 64-bit literals
-      repsuffix I64 | cINT_SIZE == 4 = ptext SLIT("L")
-      repsuffix _ = empty
+      repsuffix I64 | cINT_SIZE == 4 = ptext SLIT("UL")
+      repsuffix _ = char 'U'
       
       go 0 = empty
       go w' = go q <> dig
