@@ -1,6 +1,6 @@
 {-# OPTIONS -#include "Linker.h" #-}
 -----------------------------------------------------------------------------
--- $Id: InteractiveUI.hs,v 1.186.2.6 2005/03/08 10:05:15 simonmar Exp $
+-- $Id: InteractiveUI.hs,v 1.186.2.10 2005/07/06 10:44:44 simonmar Exp $
 --
 -- GHC Interactive User Interface
 --
@@ -28,7 +28,8 @@ import Linker		( showLinkerState, linkPackages )
 import Util
 import Name		( Name, NamedThing(..) )
 import OccName		( OccName, parenSymOcc, occNameUserString )
-import BasicTypes	( StrictnessMark(..), defaultFixity, SuccessFlag(..) )
+import BasicTypes	( StrictnessMark(..), defaultFixity, SuccessFlag(..),
+			  failed )
 import Outputable
 import CmdLineOpts	( DynFlags(..), DynFlag(..), dopt )
 import Panic 		hiding ( showException )
@@ -41,6 +42,8 @@ import System.Posix
 #if __GLASGOW_HASKELL__ > 504
 	hiding (getEnv)
 #endif
+#else
+import GHC.ConsoleHandler ( flushConsole )
 #endif
 
 #ifdef USE_READLINE
@@ -57,6 +60,7 @@ import Data.Dynamic
 import Numeric
 import Data.List
 import Data.Int		( Int64 )
+import Data.Maybe	( isJust )
 import System.Cmd
 import System.CPUTime
 import System.Environment
@@ -95,7 +99,7 @@ builtin_commands = [
   ("help",	keepGoing help),
   ("?",		keepGoing help),
   ("info",      keepGoing info),
-  ("load",	keepGoingPaths loadModule),
+  ("load",	keepGoingPaths loadModule_),
   ("module",	keepGoing setContext),
   ("reload",	keepGoing reloadModule),
   ("set",	keepGoing setCmd),
@@ -187,6 +191,13 @@ interactiveUI dflags srcs maybe_expr = do
    Readline.initialize
 #endif
 
+#if defined(mingw32_HOST_OS)
+    -- The win32 Console API mutates the first character of 
+    -- type-ahead when reading from it in a non-buffered manner. Work
+    -- around this by flushing the input buffer of type-ahead characters.
+    -- 
+   GHC.ConsoleHandler.flushConsole stdin
+#endif
    startGHCi (runGHCi srcs dflags maybe_expr)
 	GHCiState{ progname = "<interactive>",
 		   args = [],
@@ -234,9 +245,13 @@ runGHCi paths dflags maybe_expr = do
   		  Right hdl -> fileLoop hdl False
 
   -- Perform a :load for files given on the GHCi command line
-  when (not (null paths)) $
-     ghciHandle showException $
-	loadModule paths
+  -- When in -e mode, if the load fails then we want to stop
+  -- immediately rather than going on to evaluate the expression.
+  when (not (null paths)) $ do
+     ok <- ghciHandle (\e -> do showException e; return Failed) $ 
+		loadModule paths
+     when (isJust maybe_expr && failed ok) $
+	io (exitWith (ExitFailure 1))
 
   -- if verbosity is greater than 0, or we are connected to a
   -- terminal, display the prompt in the interactive loop.
@@ -259,7 +274,11 @@ runGHCi paths dflags maybe_expr = do
 interactiveLoop is_tty show_prompt = do
   -- Ignore ^C exceptions caught here
   ghciHandleDyn (\e -> case e of 
-			Interrupted -> ghciUnblock (interactiveLoop is_tty show_prompt)
+			Interrupted -> ghciUnblock (
+#if defined(mingw32_HOST_OS)
+						io (putStrLn "") >> 
+#endif
+						interactiveLoop is_tty show_prompt)
 			_other      -> return ()) $ do
 
   -- read commands from stdin
@@ -358,6 +377,11 @@ readlineLoop = do
 
 runCommand :: String -> GHCi Bool
 runCommand c = ghciHandle handler (doCommand c)
+  where 
+    doCommand (':' : command) = specialCommand command
+    doCommand stmt
+       = do timeIt (do nms <- runStmt stmt; finishEvalExpr nms)
+            return False
 
 -- This version is for the GHC command-line option -e.  The only difference
 -- from runCommand is that it catches the ExitException exception and
@@ -368,6 +392,14 @@ runCommandEval c = ghciHandle handleEval (doCommand c)
     handleEval e                    = do showException e
 				         io (exitWith (ExitFailure 1))
 
+    doCommand (':' : command) = specialCommand command
+    doCommand stmt
+       = do nms <- runStmt stmt
+	    case nms of 
+		Nothing -> io (exitWith (ExitFailure 1))
+		  -- failure to run the command causes exit(1) for ghc -e.
+		_       -> finishEvalExpr nms
+
 -- This is the exception handler for exceptions generated by the
 -- user's code; it normally just prints out the exception.  The
 -- handler must be recursive, in case showing the exception causes
@@ -394,14 +426,9 @@ showException (DynException dyn) =
 showException other_exception
   = io (putStrLn ("*** Exception: " ++ show other_exception))
 
-doCommand (':' : command) = specialCommand command
-doCommand stmt
-   = do timeIt (do nms <- runStmt stmt; finishEvalExpr nms)
-        return False
-
-runStmt :: String -> GHCi [Name]
+runStmt :: String -> GHCi (Maybe [Name])
 runStmt stmt
- | null (filter (not.isSpace) stmt) = return []
+ | null (filter (not.isSpace) stmt) = return (Just [])
  | otherwise
  = do st <- getGHCiState
       cmstate <- getCmState
@@ -410,15 +437,17 @@ runStmt stmt
 	     cmRunStmt cmstate stmt
       setGHCiState st{cmstate = new_cmstate}
       case result of
-	CmRunFailed      -> return []
+	CmRunFailed      -> return Nothing
 	CmRunException e -> throw e  -- this is caught by runCommand(Eval)
-	CmRunOk names    -> return names
+	CmRunOk names    -> return (Just names)
 
 -- possibly print the type and revert CAFs after evaluating an expression
-finishEvalExpr names
+finishEvalExpr mb_names
  = do b <- isOptionSet ShowType
       cmstate <- getCmState
-      when b (mapM_ (showTypeOfName cmstate) names)
+      case mb_names of
+	Nothing    -> return ()      
+	Just names -> when b (mapM_ (showTypeOfName cmstate) names)
 
       flushInterpBuffers
       io installSignalHandlers
@@ -697,10 +726,13 @@ undefineMacro macro_name = do
   io (writeIORef commands (filter ((/= macro_name) . fst) cmds))
 
 
-loadModule :: [FilePath] -> GHCi ()
+loadModule :: [FilePath] -> GHCi SuccessFlag
 loadModule fs = timeIt (loadModule' fs)
 
-loadModule' :: [FilePath] -> GHCi ()
+loadModule_ :: [FilePath] -> GHCi ()
+loadModule_ fs = do loadModule fs; return ()
+
+loadModule' :: [FilePath] -> GHCi SuccessFlag
 loadModule' files = do
   state <- getGHCiState
 
@@ -722,6 +754,7 @@ loadModule' files = do
   setContextAfterLoad mods
   dflags <- getDynFlags
   modulesLoadedMsg ok mods dflags
+  return ok
 
 
 reloadModule :: String -> GHCi ()
