@@ -38,7 +38,6 @@ import Module
 import ErrUtils
 import CmdLineOpts
 import Config
-import RdrName		( GlobalRdrEnv )
 import Panic
 import Util
 import StringBuffer	( hGetStringBuffer )
@@ -300,7 +299,7 @@ link Batch dflags batch_attempt_linking hpt
 	    pkg_deps  = concatMap (dep_pkgs . mi_deps . hm_iface) home_mod_infos
 
 	    -- the linkables to link
-	    linkables = map hm_linkable home_mod_infos
+	    linkables = map (fromJust.hm_linkable) home_mod_infos
 
         when (verb >= 3) $ do
 	     hPutStrLn stderr "link: linkables are ..."
@@ -471,7 +470,7 @@ genOutputFilenameFunc stop_phase keep_final_output maybe_output_filename basenam
 
 		odir_persistent
 		   | Just loc <- maybe_location = ml_obj_file loc
-		   | Just d <- odir = replaceFilenameDirectory persistent d
+		   | Just d <- odir = d ++ '/':persistent
 		   | otherwise      = persistent
 
    return func
