@@ -68,24 +68,22 @@ import Panic        (panic)
 import Outputable   (Outputable(ppr), pprPanic)
 import UniqSupply   (UniqSupply, splitUniqSupply, uniqFromSupply)
 import OccName	    (UserFS)
-import Var          (Var(..))
+import Var          (Var, idType)
 import Id	    (Id, mkSysLocal)
 import Name	    (Name)
 import VarSet       (VarSet, emptyVarSet, extendVarSet, varSetElems )
 import VarEnv       (VarEnv, emptyVarEnv, zipVarEnv, plusVarEnv,
 		     elemVarEnv, lookupVarEnv, lookupVarEnv_NF, delVarEnvList)
-import TyCon        (tyConName)
 import Type	    (Type, tyConAppTyCon)
-import HscTypes	    (HomePackageTable, PersistentCompilerState(pcs_EPS), 
+import HscTypes	    (HomePackageTable,
 		     ExternalPackageState(eps_PTE), HscEnv(hsc_HPT),
 		     TyThing(..), lookupType)
-import PrelNames    (charPrimTyConName, intPrimTyConName, floatPrimTyConName,
-		     doublePrimTyConName, fstName, andName, orName,
+import PrelNames    ( fstName, andName, orName,
 		     lengthPName, replicatePName, mapPName, bpermutePName,
 		     bpermuteDftPName, indexOfPName)
-import PrimOp	    (eqCharName, eqIntName, eqFloatName, eqDoubleName,
-		     neqIntName)
-		     -- neqCharName, neqFloatName,neqDoubleName,
+import TysPrim      ( charPrimTyCon, intPrimTyCon, floatPrimTyCon, doublePrimTyCon )
+import PrimOp	    ( PrimOp(..) )
+import PrelInfo	    ( primOpId )
 import CoreSyn      (Expr(..), Bind(..), CoreBndr, CoreExpr, CoreBind, mkApps)
 import CoreUtils    (exprType)
 
@@ -130,11 +128,11 @@ data FlattenState = FlattenState {
 
 -- initial value of the flattening state
 --
-initialFlattenState :: PersistentCompilerState 
+initialFlattenState :: ExternalPackageState
 		    -> HomePackageTable 
 		    -> UniqSupply 
 		    -> FlattenState
-initialFlattenState pcs hpt us = 
+initialFlattenState eps hpt us = 
   FlattenState {
     us	     = us,
     env      = lookup,
@@ -144,7 +142,7 @@ initialFlattenState pcs hpt us =
   }
   where
     lookup n = 
-      case lookupType hpt (eps_PTE (pcs_EPS pcs)) n of
+      case lookupType hpt (eps_PTE eps) n of
         Just (AnId v) -> v 
 	_             -> pprPanic "FlattenMonad: unknown name:" (ppr n)
 
@@ -164,12 +162,12 @@ instance Monad Flatten where
 -- execute the given flattening computation (EXPORTED)
 --
 runFlatten :: HscEnv
-	   -> PersistentCompilerState 
+	   -> ExternalPackageState
 	   -> UniqSupply 
 	   -> Flatten a 
 	   -> a    
-runFlatten hsc_env pcs us m 
-  = fst $ unFlatten m (initialFlattenState pcs (hsc_HPT hsc_env) us)
+runFlatten hsc_env eps us m 
+  = fst $ unFlatten m (initialFlattenState eps (hsc_HPT hsc_env) us)
 
 
 -- variable generation
@@ -276,7 +274,7 @@ packContext perm m  = Flatten $ \state ->
     -- generate a binding for the packed variant of a context variable
     --
     mkCoreBind var    = let
-			  rhs = fst $ unFlatten (mk'bpermuteP (varType var) 
+			  rhs = fst $ unFlatten (mk'bpermuteP (idType var) 
 							      (Var perm) 
 							      (Var var)
 						) state
@@ -303,9 +301,9 @@ liftVar     :: Var -> Flatten CoreExpr
 liftVar var  = Flatten $ \s ->
   let 
     v          = ctxtVarErr s
-    v'elemType = parrElemTy . varType $ v
+    v'elemType = parrElemTy . idType $ v
     len        = fst $ unFlatten (mk'lengthP v'elemType (Var v)) s
-    replicated = fst $ unFlatten (mk'replicateP (varType var) len (Var var)) s
+    replicated = fst $ unFlatten (mk'replicateP (idType var) len (Var var)) s
   in case lookupVarEnv (ctxtEnv s) var of
     Just liftedVar -> (Var liftedVar, 
 		       s {usedVars = usedVars s `extendVarSet` var})
@@ -320,7 +318,7 @@ liftConst   :: CoreExpr -> Flatten CoreExpr
 liftConst e  = Flatten $ \s ->
   let
      v          = ctxtVarErr s
-     v'elemType = parrElemTy . varType $ v
+     v'elemType = parrElemTy . idType $ v
      len        = fst $ unFlatten (mk'lengthP v'elemType (Var v)) s
   in 
   (fst $ unFlatten (mk'replicateP (exprType e) len e ) s, s)
@@ -364,14 +362,14 @@ mk'or a1 a2  = mkFunApp orName [a1, a2]
 -- `Double') (EXPORTED)
 --
 mk'eq          :: Type -> CoreExpr -> CoreExpr -> Flatten CoreExpr
-mk'eq ty a1 a2  = mkFunApp eqName [a1, a2]
+mk'eq ty a1 a2  = return (mkApps (Var eqName) [a1, a2])
 		  where
-		    name = tyConName . tyConAppTyCon $ ty
+		    tc = tyConAppTyCon ty
 		    --
-		    eqName | name == charPrimTyConName   = eqCharName
-			   | name == intPrimTyConName    = eqIntName
-			   | name == floatPrimTyConName  = eqFloatName
-			   | name == doublePrimTyConName = eqDoubleName
+		    eqName | tc == charPrimTyCon   = primOpId CharEqOp
+			   | tc == intPrimTyCon    = primOpId IntEqOp
+			   | tc == floatPrimTyCon  = primOpId FloatEqOp
+			   | tc == doublePrimTyCon = primOpId DoubleEqOp
 			   | otherwise 		         =
 			     pprPanic "FlattenMonad.mk'eq: " (ppr ty)
 
@@ -380,12 +378,12 @@ mk'eq ty a1 a2  = mkFunApp eqName [a1, a2]
 -- `Double') (EXPORTED)
 --
 mk'neq          :: Type -> CoreExpr -> CoreExpr -> Flatten CoreExpr
-mk'neq ty a1 a2  = mkFunApp neqName [a1, a2]
+mk'neq ty a1 a2  = return (mkApps (Var neqName) [a1, a2])
 		   where
-		     name = tyConName . tyConAppTyCon $ ty
+		     tc = tyConAppTyCon ty
 		     --
 		     neqName {- | name == charPrimTyConName   = neqCharName -}
-			     | name == intPrimTyConName    = neqIntName
+			     | tc == intPrimTyCon	      = primOpId IntNeOp
 			     {- | name == floatPrimTyConName  = neqFloatName -}
 			     {- | name == doublePrimTyConName = neqDoubleName -}
 			     | otherwise		   =
