@@ -1,5 +1,5 @@
 /* ----------------------------------------------------------------------------
- * $Id: Constants.h,v 1.27 2004/11/18 09:56:19 tharris Exp $
+ * $Id: Constants.h,v 1.27.6.1 2005/05/12 09:32:15 simonmar Exp $
  *
  * (c) The GHC Team, 1998-2002
  *
@@ -267,4 +267,10 @@
 #error RESERVED_STACK_WORDS may be wrong!
 #endif
 
+/* -----------------------------------------------------------------------------
+   How often our context-switch timer ticks
+   -------------------------------------------------------------------------- */
+
+#define TICK_FREQUENCY   50                      /* ticks per second */
+
 #endif /* CONSTANTS_H */
