@@ -1,7 +1,6 @@
 /* ----------------------------------------------------------------------------
- * $Id: RtsAPI.h,v 1.22.4.1 2001/10/30 11:19:47 sewardj Exp $
  *
- * (c) The GHC Team, 1998-1999
+ * (c) The GHC Team, 1998-2004
  *
  * API for invoking Haskell functions via the RTS
  *
@@ -23,8 +22,7 @@ typedef enum {
     NoStatus,    /* not finished yet */
     Success,	 /* completed successfully */
     Killed,	 /* uncaught exception */
-    Interrupted, /* stopped in response to a call to interruptStgRts */
-    Deadlock	 /* no threads to run, but main thread hasn't finished */
+    Interrupted  /* stopped in response to a call to interruptStgRts */
 } SchedulerStatus;
 
 typedef StgClosure *HaskellObj;
@@ -36,8 +34,22 @@ extern void startupHaskell         ( int argc, char *argv[],
 				     void (*init_root)(void) );
 extern void shutdownHaskell        ( void );
 extern void shutdownHaskellAndExit ( int exitCode );
-extern void setProgArgv            ( int argc, char *argv[] );
 extern void getProgArgv            ( int *argc, char **argv[] );
+extern void setProgArgv            ( int argc, char *argv[] );
+
+
+/* ----------------------------------------------------------------------------
+   Locking.
+   
+   In a multithreaded environments, you have to surround all access to the
+   RtsAPI with these calls.
+   ------------------------------------------------------------------------- */
+   
+void
+rts_lock ( void );
+
+void
+rts_unlock ( void );
 
 /* ----------------------------------------------------------------------------
    Building Haskell objects from C datatypes.
@@ -54,6 +66,7 @@ HaskellObj   rts_mkWord16     ( HsWord16 w );
 HaskellObj   rts_mkWord32     ( HsWord32 w );
 HaskellObj   rts_mkWord64     ( HsWord64 w );
 HaskellObj   rts_mkPtr        ( HsPtr    a );
+HaskellObj   rts_mkFunPtr     ( HsFunPtr a );
 HaskellObj   rts_mkFloat      ( HsFloat  f );
 HaskellObj   rts_mkDouble     ( HsDouble f );
 HaskellObj   rts_mkStablePtr  ( HsStablePtr s );
@@ -62,9 +75,6 @@ HaskellObj   rts_mkString     ( char    *s );
 
 HaskellObj   rts_apply        ( HaskellObj, HaskellObj );
 
-/* DEPRECATED (use rts_mkPtr): */
-HaskellObj   rts_mkAddr       ( HsAddr   a );
-
 /* ----------------------------------------------------------------------------
    Deconstructing Haskell objects
    ------------------------------------------------------------------------- */
@@ -80,18 +90,18 @@ HsWord16     rts_getWord16    ( HaskellObj );
 HsWord32     rts_getWord32    ( HaskellObj );
 HsWord64     rts_getWord64    ( HaskellObj );
 HsPtr        rts_getPtr       ( HaskellObj );
+HsFunPtr     rts_getFunPtr    ( HaskellObj );
 HsFloat      rts_getFloat     ( HaskellObj );
 HsDouble     rts_getDouble    ( HaskellObj );
 HsStablePtr  rts_getStablePtr ( HaskellObj );
 HsBool       rts_getBool      ( HaskellObj );
 
-/* DEPRECATED (use rts_getPtr): */
-HsAddr       rts_getAddr      ( HaskellObj );
-
 /* ----------------------------------------------------------------------------
    Evaluating Haskell expressions
 
    The versions ending in '_' allow you to specify an initial stack size.
+   Note that these calls may cause Garbage Collection, so all HaskellObj
+   references are rendered invalid by these calls.
    ------------------------------------------------------------------------- */
 SchedulerStatus 
 rts_eval ( HaskellObj p, /*out*/HaskellObj *ret );
@@ -102,13 +112,30 @@ rts_eval_ ( HaskellObj p, unsigned int stack_size, /*out*/HaskellObj *ret );
 SchedulerStatus 
 rts_evalIO ( HaskellObj p, /*out*/HaskellObj *ret );
 
+SchedulerStatus
+rts_evalStableIO ( HsStablePtr s, /*out*/HsStablePtr *ret );
+
 SchedulerStatus 
-rts_evalLazyIO ( HaskellObj p, unsigned int stack_size, /*out*/HaskellObj *ret );
+rts_evalLazyIO ( HaskellObj p, /*out*/HaskellObj *ret );
+
+SchedulerStatus 
+rts_evalLazyIO_ ( HaskellObj p, unsigned int stack_size, /*out*/HaskellObj *ret );
 
 void
 rts_checkSchedStatus ( char* site, SchedulerStatus rc);
 
-/* -------------------------------------------------------------------------- */
+/* --------------------------------------------------------------------------
+   Wrapper closures
+
+   These are used by foreign export and foreign import "wrapper" stubs.
+   ----------------------------------------------------------------------- */
+
+extern StgWord GHCziTopHandler_runIO_closure[];
+extern StgWord GHCziTopHandler_runNonIO_closure[];
+#define runIO_closure		  GHCziTopHandler_runIO_closure
+#define runNonIO_closure	  GHCziTopHandler_runNonIO_closure
+
+/* ------------------------------------------------------------------------ */
 
 #ifdef __cplusplus
 }
