@@ -222,8 +222,8 @@ thread_arg_block (StgFunInfoTable *fun_info, StgClosure **args)
     p = (StgPtr)args;
     switch (fun_info->f.fun_type) {
     case ARG_GEN:
-	bitmap = BITMAP_BITS(fun_info->f.bitmap);
-	size = BITMAP_SIZE(fun_info->f.bitmap);
+	bitmap = BITMAP_BITS(fun_info->f.b.bitmap);
+	size = BITMAP_SIZE(fun_info->f.b.bitmap);
 	goto small_bitmap;
     case ARG_GEN_BIG:
 	size = GET_FUN_LARGE_BITMAP(fun_info)->size;
@@ -378,7 +378,7 @@ thread_PAP (StgPAP *pap)
 
     switch (fun_info->f.fun_type) {
     case ARG_GEN:
-	bitmap = BITMAP_BITS(fun_info->f.bitmap);
+	bitmap = BITMAP_BITS(fun_info->f.b.bitmap);
 	goto small_bitmap;
     case ARG_GEN_BIG:
 	thread_large_bitmap(p, GET_FUN_LARGE_BITMAP(fun_info), size);
@@ -484,6 +484,20 @@ update_fwd_large( bdescr *bd )
 	thread_PAP((StgPAP *)p);
 	continue;
 
+    case TREC_CHUNK:
+    {
+        StgWord i;
+        StgTRecChunk *tc = (StgTRecChunk *)p;
+	TRecEntry *e = &(tc -> entries[0]);
+	thread((StgPtr)&tc->prev_chunk);
+	for (i = 0; i < tc -> next_entry_idx; i ++, e++ ) {
+	  thread((StgPtr)&e->tvar);
+	  thread((StgPtr)&e->expected_value);
+	  thread((StgPtr)&e->new_value);
+	}
+	continue;
+    }
+
     default:
       barf("update_fwd_large: unknown/strange object  %d", (int)(info->type));
     }
@@ -538,7 +552,6 @@ thread_obj (StgInfoTable *info, StgPtr p)
     case FUN:
     case THUNK:
     case CONSTR:
-    case FOREIGN:
     case STABLE_NAME:
     case IND_PERM:
     case MUT_VAR:
@@ -839,11 +852,6 @@ update_bkwd_compact( step *stp )
 		move(free,p,size);
 	    }
 
-	    // Rebuild the mutable list for the old generation.
-	    if (ip_MUTABLE(info)) {
-		recordMutable((StgMutClosure *)free);
-	    }
-
 	    // relocate TSOs
 	    if (info->type == TSO) {
 		move_TSO((StgTSO *)p, (StgTSO *)free);
@@ -869,16 +877,14 @@ update_bkwd_compact( step *stp )
 }
 
 static void
-thread_mut_once_list( generation *g )
+thread_mut_list( StgMutClosure *q )
 {
     StgMutClosure *p, *next;
 
-    for (p = g->mut_once_list; p != END_MUT_LIST; p = next) {
+    for (p = q; p != END_MUT_LIST; p = next) {
 	next = p->mut_link;
 	thread((StgPtr)&p->mut_link);
     }
-    
-    thread((StgPtr)&g->mut_once_list);
 }
 
 void
@@ -900,8 +906,10 @@ compact( void (*get_roots)(evac_fn) )
 
     // mutable lists
     for (g = 1; g < RtsFlags.GcFlags.generations; g++) {
+	thread_mut_list(generations[g].mut_list);
 	thread((StgPtr)&generations[g].mut_list);
-	thread_mut_once_list(&generations[g]);
+	thread_mut_list(generations[g].mut_once_list);
+	thread((StgPtr)&generations[g].mut_once_list);
     }
 
     // the global thread list
