@@ -169,7 +169,7 @@ checkStackFrame( StgPtr c )
 	switch (fun_info->f.fun_type) {
 	case ARG_GEN:
 	    checkSmallBitmap((StgPtr)ret_fun->payload, 
-			     BITMAP_BITS(fun_info->f.bitmap), size);
+			     BITMAP_BITS(fun_info->f.b.bitmap), size);
 	    break;
 	case ARG_GEN_BIG:
 	    checkLargeBitmap((StgPtr)ret_fun->payload,
@@ -327,12 +327,9 @@ checkClosure( StgClosure* p )
   	    /* we don't expect to see any of these after GC
 	     * but they might appear during execution
 	     */
-	    P_ q;
 	    StgInd *ind = (StgInd *)p;
 	    ASSERT(LOOKS_LIKE_CLOSURE_PTR(ind->indirectee));
-	    q = (P_)p + sizeofW(StgInd);
-	    while (!*q) { q++; }; /* skip padding words (see GC.c: evacuate())*/
-	    return q - (P_)p;
+	    return sizeofW(StgHeader) + MIN_UPD_SIZE;
 	}
 
     case RET_BCO:
@@ -362,7 +359,7 @@ checkClosure( StgClosure* p )
 	    switch (fun_info->f.fun_type) {
 	    case ARG_GEN:
 		checkSmallBitmap( (StgPtr)pap->payload, 
-				  BITMAP_BITS(fun_info->f.bitmap), pap->n_args );
+				  BITMAP_BITS(fun_info->f.b.bitmap), pap->n_args );
 		break;
 	    case ARG_GEN_BIG:
 		checkLargeBitmap( (StgPtr)pap->payload, 
@@ -907,26 +904,23 @@ checkBQ (StgTSO *bqe, StgClosure *closure)
 void
 checkBQ (StgTSO *bqe, StgClosure *closure) 
 {  
-  rtsBool end = rtsFalse;
   StgInfoTable *info = get_itbl(closure);
 
   ASSERT(info->type == BLACKHOLE_BQ || info->type == MVAR);
 
-  do {
-    switch (get_itbl(bqe)->type) {
-    case TSO:
-      checkClosure((StgClosure *)bqe);
-      bqe = bqe->link;
-      end = (bqe==END_TSO_QUEUE);
-      break;
-
-    default:
-      barf("checkBQ: strange closure %d in blocking queue for closure %p\n", 
-	   get_itbl(bqe)->type, closure, info->type);
-    }
-  } while (!end);
+  while (bqe != END_TSO_QUEUE) {    
+      switch (get_itbl(bqe)->type) {
+      case TSO:
+	  checkClosure((StgClosure *)bqe);
+	  bqe = bqe->link;
+	  continue;
+      default:
+	  barf("checkBQ: strange closure %d in blocking queue for closure %p\n", 
+	       get_itbl(bqe)->type, closure, info->type);
+      }
+  }
 }
-    
+
 #endif
     
 
@@ -981,4 +975,15 @@ checkLAGAtable(rtsBool check_closures)
 }
 #endif
 
+int
+onMutList(StgMutClosure *p, StgMutClosure *mut)
+{
+    int i;
+    for (i=0; mut != END_MUT_LIST; i++) {
+	if (mut == p) return i;
+	mut = mut->mut_link;
+    }
+    return -1;
+}
+
 #endif /* DEBUG */
