@@ -1,5 +1,4 @@
 /* -----------------------------------------------------------------------------
- * $Id: Weak.c,v 1.29.6.2 2003/11/10 12:03:52 simonmar Exp $
  *
  * (c) The GHC Team, 1998-1999
  *
@@ -10,6 +9,7 @@
 #include "PosixSource.h"
 #define COMPILING_RTS_MAIN
 #include "Rts.h"
+#include "RtsUtils.h"
 #include "SchedAPI.h"
 #include "RtsFlags.h"
 #include "Weak.h"
@@ -20,37 +20,6 @@
 
 StgWeak *weak_ptr_list;
 
-/*
- * finalizeWeakPointersNow() is called just before the system is shut
- * down.  It runs the finalizer for each weak pointer still in the
- * system.
- *
- * Careful here - rts_evalIO might cause a garbage collection, which
- * might change weak_ptr_list.  Must re-load weak_ptr_list each time
- * around the loop.
- */
-
-void
-finalizeWeakPointersNow(void)
-{
-  StgWeak *w;
-  
-  rts_lock();
-  while ((w = weak_ptr_list)) {
-    weak_ptr_list = w->link;
-    if (w->header.info != &stg_DEAD_WEAK_info) {
-	w->header.info = &stg_DEAD_WEAK_info;
-	IF_DEBUG(weak,fprintf(stderr,"Finalising weak pointer at %p -> %p\n", w, w->key));
-	if (w->finalizer != &stg_NO_FINALIZER_closure) {
-	    rts_evalLazyIO(w->finalizer,NULL);
-	    rts_unlock();
-	    rts_lock();
-	}
-    }
-  }
-  rts_unlock();
-} 
-
 /*
  * scheduleFinalizers() is called on the list of weak pointers found
  * to be dead after a garbage collection.  It overwrites each object
@@ -101,7 +70,7 @@ scheduleFinalizers(StgWeak *list)
     // No finalizers to run?
     if (n == 0) return;
 
-    IF_DEBUG(weak,fprintf(stderr,"weak: batching %d finalizers\n", n));
+    IF_DEBUG(weak,debugBelch("weak: batching %d finalizers\n", n));
 
     arr = (StgMutArrPtrs *)allocate(sizeofW(StgMutArrPtrs) + n);
     TICK_ALLOC_PRIM(sizeofW(StgMutArrPtrs), n, 0);
