@@ -1,11 +1,16 @@
-
+{-# OPTIONS -cpp #-}
 ------------------------------------------------------------------
 -- A primop-table mangling program                              --
 ------------------------------------------------------------------
 
 module Main where
 
+#if __GLASGOW_HASKELL__ >= 504
+import Text.ParserCombinators.Parsec
+#else
 import Parsec
+#endif
+
 import Monad
 import Char
 import List
@@ -249,15 +254,18 @@ gen_latex_doc (Info defaults entries)
 	   latex_encode (c:cs) = c:(latex_encode cs)
 
 gen_wrappers (Info defaults entries)
-   = "module PrelPrimopWrappers where\n" 
-     ++ "import qualified PrelGHC\n" 
+   = "{-# OPTIONS -fno-implicit-prelude #-}\n" 
+	-- Dependencies on Prelude must be explicit in libraries/base, but we
+	-- don't need the Prelude here so we add -fno-implicit-prelude.
+     ++ "module GHC.PrimopWrappers where\n" 
+     ++ "import qualified GHC.Prim\n" 
      ++ unlines (map f (filter (not.dodgy) (filter is_primop entries)))
      where
         f spec = let args = map (\n -> "a" ++ show n) [1 .. arity (ty spec)]
                      src_name = wrap (name spec)
                  in "{-# NOINLINE " ++ src_name ++ " #-}\n" ++ 
                     src_name ++ " " ++ unwords args 
-                     ++ " = (PrelGHC." ++ name spec ++ ") " ++ unwords args
+                     ++ " = (GHC.Prim." ++ name spec ++ ") " ++ unwords args
         wrap nm | isLower (head nm) = nm
                 | otherwise = "(" ++ nm ++ ")"
 
@@ -282,10 +290,12 @@ gen_primop_list (Info defaults entries)
      ) where (first:rest) = filter is_primop entries
 
 gen_primop_tag (Info defaults entries)
-   = unlines (zipWith f (filter is_primop entries) [1..])
+   = unlines (max_def : zipWith f primop_entries [1..])
      where
+	primop_entries = filter is_primop entries
         f i n = "tagOf_PrimOp " ++ cons i 
                 ++ " = _ILIT(" ++ show n ++ ") :: FastInt"
+	max_def = "maxPrimOpTag = " ++ show (length primop_entries) ++ " :: Int"
 
 gen_data_decl (Info defaults entries)
    = let conss = map cons (filter is_primop entries)
@@ -349,7 +359,7 @@ mkPOI_RHS_text i
                       ++ listify (map ppType argTys) ++ " "
                       ++ "(" ++ ppType resTy ++ ")"
             
-sl_name i = "SLIT(\"" ++ name i ++ "\") "
+sl_name i = "FSLIT(\"" ++ name i ++ "\") "
 
 ppTyVar "a" = "alphaTyVar"
 ppTyVar "b" = "betaTyVar"
@@ -375,7 +385,7 @@ ppType (TyApp "RealWorld"   []) = "realWorldTy"
 ppType (TyApp "ThreadId#"   []) = "threadIdPrimTy"
 ppType (TyApp "ForeignObj#" []) = "foreignObjPrimTy"
 ppType (TyApp "BCO#"        []) = "bcoPrimTy"
-ppType (TyApp "Unit"        []) = "unitTy"   -- dodgy
+ppType (TyApp "()"          []) = "unitTy" 	-- unitTy is TysWiredIn's name for ()
 
 
 ppType (TyVar "a")               = "alphaTy"
@@ -401,6 +411,8 @@ ppType (TyApp "StableName#"  [x])      = "mkStableNamePrimTy " ++ ppType x
 
 ppType (TyApp "MVar#" [x,y])     = "mkMVarPrimTy " ++ ppType x 
                                    ++ " " ++ ppType y
+ppType (TyApp "TVar#" [x,y])     = "mkTVarPrimTy " ++ ppType x 
+                                   ++ " " ++ ppType y
 ppType (TyUTup ts)               = "(mkTupleTy Unboxed " ++ show (length ts)
                                    ++ " "
                                    ++ listify (map ppType ts) ++ ")"
@@ -639,7 +651,7 @@ ppT = alts [apply TyVar pTyvar,
            ]
 
 pTyvar       = sat (`notElem` ["section","primop","with"]) pName
-pTycon       = pConstructor
+pTycon       = alts [pConstructor, lexeme (string "()")]
 pName        = lexeme (then2 (:) lower (many isIdChar))
 pConstructor = lexeme (then2 (:) upper (many isIdChar))
 
