@@ -1,4 +1,6 @@
-module ClCrossCall_12 ( module ClCrossCall_12 ) where
+module ClCrossCall_12
+	{-# DEPRECATED "This module has been superseded by Graphics.UI.ObjectIO" #-}
+	( module ClCrossCall_12 ) where
 
 
 --	********************************************************************************
