@@ -1,4 +1,6 @@
-module ControlLayout ( layoutControls, calcControlsSize ) where
+module ControlLayout
+	{-# DEPRECATED "This module has been superseded by Graphics.UI.ObjectIO" #-}
+	( layoutControls, calcControlsSize ) where
 
 
 --	********************************************************************************
