@@ -1,4 +1,6 @@
-module ReceiverId where
+module ReceiverId
+	{-# DEPRECATED "This module has been superseded by Graphics.UI.ObjectIO" #-}
+	where
 
 
 --	********************************************************************************
