@@ -1,4 +1,6 @@
-module StdControlAttribute (module StdControlAttribute, module StdControlDef) where
+module StdControlAttribute
+	{-# DEPRECATED "This module has been superseded by Graphics.UI.ObjectIO" #-}
+	(module StdControlAttribute, module StdControlDef) where
 
 
 --	********************************************************************************
