@@ -1,4 +1,6 @@
-module StdControlDef ( module StdControlDef
+module StdControlDef
+                     {-# DEPRECATED "This module has been superseded by Graphics.UI.ObjectIO" #-}
+                     ( module StdControlDef
                      , module StdIOCommon
                      , module StdGUI
                      ) where
