@@ -1,4 +1,6 @@
-module WindowValidate ( validateWindowId
+module WindowValidate
+                      {-# DEPRECATED "This module has been superseded by Graphics.UI.ObjectIO" #-}
+                      ( validateWindowId
                       , validateWindow
                       , exactWindowSize
                       , exactWindowPos
