@@ -6,20 +6,6 @@
 #ifndef _FINALIZERS_H
 #define _FINALIZERS_H
 
-#ifdef TARGET_GHC
 extern void deleteObj(HANDLE h);
-#else
-static
-void
-deleteObj(HANDLE h)
-{
-  if (h == NULL)
-    return;
-  else {
-    DeleteObject(h);
-  }
-}
-
-#endif
 
 #endif /* _FINALIZERS_H */
