@@ -1,5 +1,5 @@
 /* -----------------------------------------------------------------------------
- * $Id: HsXlib.h,v 1.1.2.1 2001/12/17 11:18:36 simonmar Exp $
+ * $Id: HsXlib.h,v 1.3 2002/06/22 16:52:45 reid Exp $
  *
  * Definitions for package `lang' which are visible in Haskell land.
  *
@@ -25,6 +25,12 @@
 #define BadStatus(err,where) ErrorMsg(where,"bad status")
 #define Zero(err,where)      ErrorMsg(where,"zero")
 
+/* AllPlanes is a macro so we can't call it.
+ * For now we can get round this by defining a macro that looks like
+ * it is a function.  (Slightly illegal under the ffi spec.)
+ */
+#define AllPlanes_aux() AllPlanes
+
 /* This error handler is used from GreenCard code.
  * It generates a slightly better error message than the one
  * that comes with Xlib.
