@@ -168,6 +168,7 @@ register pkg_descr lbi (userInst, genScript, verbose)
 	let allFlags = "--auto-ghci-libs":
 		       (register_flags
                         ++ config_flags)
+                       ++ if ghc_63_plus && genScript then ["-"] else []
         let pkgTool = compilerPkgTool (compiler lbi)
 
         if genScript
@@ -236,7 +237,7 @@ mkInstalledPackageInfo pkg_descr lbi
 	IPI.exposedModules    = exposedModules lib,
 	IPI.hiddenModules     = otherModules bi,
         IPI.importDirs        = [mkLibDir pkg_descr lbi Nothing],
-        IPI.libraryDirs       = [mkLibDir pkg_descr lbi Nothing],
+        IPI.libraryDirs       = (mkLibDir pkg_descr lbi Nothing) : extraLibDirs bi,
         IPI.hsLibraries       = ["HS" ++ showPackageId (package pkg_descr)],
         IPI.extraLibraries    = extraLibs bi,
         IPI.includeDirs       = includeDirs bi,
