@@ -5,7 +5,7 @@
 -- License     :  BSD-style (see the file libraries/GLUT/LICENSE)
 -- 
 -- Maintainer  :  sven.panne@aedion.de
--- Stability   :  provisional
+-- Stability   :  stable
 -- Portability :  portable
 --
 --------------------------------------------------------------------------------
@@ -20,6 +20,9 @@ module Graphics.UI.GLUT.Callbacks.Window (
    -- * Callback for visibility changes
    Visibility(..), VisibilityCallback, visibilityCallback,
 
+   -- * Window close callback
+   CloseCallback, closeCallback,
+
    -- * Keyboard and mouse input callback
    Key(..), SpecialKey(..), MouseButton(..), KeyState(..), Modifiers(..),
    KeyboardMouseCallback, keyboardMouseCallback,
@@ -49,7 +52,6 @@ import Data.Bits ( Bits((.&.)) )
 import Data.Char ( chr )
 import Data.Maybe ( fromJust )
 import Foreign.C.Types ( CInt, CUInt, CUChar )
-import Foreign.Ptr ( FunPtr )
 import Graphics.Rendering.OpenGL.GL.CoordTrans ( Position(..), Size(..) )
 import Graphics.Rendering.OpenGL.GL.StateVar (
    SettableStateVar, makeSettableStateVar )
@@ -68,6 +70,11 @@ import Graphics.UI.GLUT.Constants (
    glut_JOYSTICK_BUTTON_C, glut_JOYSTICK_BUTTON_D )
 import Graphics.UI.GLUT.State ( PollRate )
 import Graphics.UI.GLUT.Types ( MouseButton(..), unmarshalMouseButton )
+import Graphics.UI.GLUT.Extensions
+
+--------------------------------------------------------------------------------
+
+#include "HsGLUTExt.h"
 
 --------------------------------------------------------------------------------
 
@@ -178,7 +185,7 @@ type ReshapeCallback' = CInt -> CInt -> IO ()
 -- default reshape callback is used. This default callback will simply call
 --
 -- @
--- 'viewport' ('Graphics.Rendering.OpenGL.GL.CoordTrans.Position' 0 0) ('Graphics.Rendering.OpenGL.GL.CoordTrans.Size' /width/ /height/)
+-- 'Graphics.Rendering.OpenGL.GL.CoordTrans.viewport' ('Graphics.Rendering.OpenGL.GL.CoordTrans.Position' 0 0) ('Graphics.Rendering.OpenGL.GL.CoordTrans.Size' /width/ /height/)
 -- @
 --
 -- on the normal plane (and on the overlay if one exists).
@@ -218,10 +225,10 @@ data Visibility
    deriving ( Eq, Ord, Show )
 
 unmarshalVisibility :: CInt -> Visibility
-unmarshalVisibility v
-   | v == glut_NOT_VISIBLE = NotVisible
-   | v == glut_VISIBLE     = Visible
-   | otherwise = error "unmarshalVisibility"
+unmarshalVisibility x
+   | x == glut_NOT_VISIBLE = NotVisible
+   | x == glut_VISIBLE = Visible
+   | otherwise = error ("unmarshalVisibility: illegal value " ++ show x)
 
 --------------------------------------------------------------------------------
 
@@ -253,6 +260,19 @@ foreign import CALLCONV unsafe "glutVisibilityFunc" glutVisibilityFunc ::
 
 --------------------------------------------------------------------------------
 
+type CloseCallback = IO ()
+
+closeCallback :: SettableStateVar (Maybe CloseCallback)
+closeCallback = makeSettableStateVar $
+   setCallback CloseCB glutCloseFunc makeCloseCallback
+
+foreign import ccall "wrapper"
+   makeCloseCallback :: CloseCallback -> IO (FunPtr CloseCallback)
+
+EXTENSION_ENTRY(unsafe,"freeglut",glutCloseFunc,FunPtr CloseCallback -> IO ())
+
+--------------------------------------------------------------------------------
+
 type KeyboardCallback = Char -> Position -> IO ()
 
 type KeyboardCallback' = CUChar -> CInt -> CInt -> IO ()
@@ -310,29 +330,29 @@ data SpecialKey
    deriving ( Eq, Ord, Show )
 
 unmarshalSpecialKey :: CInt -> SpecialKey
-unmarshalSpecialKey k
-   | k == glut_KEY_F1        = KeyF1
-   | k == glut_KEY_F2        = KeyF2
-   | k == glut_KEY_F3        = KeyF3
-   | k == glut_KEY_F4        = KeyF4
-   | k == glut_KEY_F5        = KeyF5
-   | k == glut_KEY_F6        = KeyF6
-   | k == glut_KEY_F7        = KeyF7
-   | k == glut_KEY_F8        = KeyF8
-   | k == glut_KEY_F9        = KeyF9
-   | k == glut_KEY_F10       = KeyF10
-   | k == glut_KEY_F11       = KeyF11
-   | k == glut_KEY_F12       = KeyF12
-   | k == glut_KEY_LEFT      = KeyLeft
-   | k == glut_KEY_UP        = KeyUp
-   | k == glut_KEY_RIGHT     = KeyRight
-   | k == glut_KEY_DOWN      = KeyDown
-   | k == glut_KEY_PAGE_UP   = KeyPageUp
-   | k == glut_KEY_PAGE_DOWN = KeyPageDown
-   | k == glut_KEY_HOME      = KeyHome
-   | k == glut_KEY_END       = KeyEnd
-   | k == glut_KEY_INSERT    = KeyInsert
-   | otherwise = error "unmarshalSpecialKey"
+unmarshalSpecialKey x
+   | x == glut_KEY_F1 = KeyF1
+   | x == glut_KEY_F2 = KeyF2
+   | x == glut_KEY_F3 = KeyF3
+   | x == glut_KEY_F4 = KeyF4
+   | x == glut_KEY_F5 = KeyF5
+   | x == glut_KEY_F6 = KeyF6
+   | x == glut_KEY_F7 = KeyF7
+   | x == glut_KEY_F8 = KeyF8
+   | x == glut_KEY_F9 = KeyF9
+   | x == glut_KEY_F10 = KeyF10
+   | x == glut_KEY_F11 = KeyF11
+   | x == glut_KEY_F12 = KeyF12
+   | x == glut_KEY_LEFT = KeyLeft
+   | x == glut_KEY_UP = KeyUp
+   | x == glut_KEY_RIGHT = KeyRight
+   | x == glut_KEY_DOWN = KeyDown
+   | x == glut_KEY_PAGE_UP = KeyPageUp
+   | x == glut_KEY_PAGE_DOWN = KeyPageDown
+   | x == glut_KEY_HOME = KeyHome
+   | x == glut_KEY_END = KeyEnd
+   | x == glut_KEY_INSERT = KeyInsert
+   | otherwise = error ("unmarshalSpecialKey: illegal value " ++ show x)
 
 --------------------------------------------------------------------------------
 
@@ -373,10 +393,10 @@ data KeyState
    deriving ( Eq, Ord, Show )
 
 unmarshalKeyState :: CInt -> KeyState
-unmarshalKeyState s
-   | s == glut_DOWN = Down
-   | s == glut_UP   = Up
-   | otherwise = error "unmarshalKeyState"
+unmarshalKeyState x
+   | x == glut_DOWN = Down
+   | x == glut_UP = Up
+   | otherwise = error ("unmarshalKeyState: illegal value " ++ show x)
 
 --------------------------------------------------------------------------------
 
@@ -510,10 +530,10 @@ data Crossing
    deriving ( Eq, Ord, Show )
 
 unmarshalCrossing :: CInt -> Crossing
-unmarshalCrossing c
-   | c == glut_LEFT    = WindowLeft
-   | c == glut_ENTERED = WindowEntered
-   | otherwise = error "unmarshalCrossing"
+unmarshalCrossing x
+   | x == glut_LEFT = WindowLeft
+   | x == glut_ENTERED = WindowEntered
+   | otherwise = error ("unmarshalCrossing: illegal value " ++ show x)
 
 --------------------------------------------------------------------------------
 
