@@ -5,7 +5,7 @@
 -- License     :  BSD-style (see the file libraries/GLUT/LICENSE)
 -- 
 -- Maintainer  :  sven.panne@aedion.de
--- Stability   :  provisional
+-- Stability   :  stable
 -- Portability :  portable
 --
 -- GLUT supports two types of windows: top-level windows and subwindows. Both
@@ -25,7 +25,7 @@ module Graphics.UI.GLUT.Window (
    parentWindow, numSubWindows,
 
    -- * Manipulating the /current window/
-   currentWindow, isRealWindow,
+   currentWindow,
 
    -- * Re-displaying and double buffer management
    postRedisplay, swapBuffers,
@@ -88,7 +88,7 @@ import Graphics.UI.GLUT.Types ( Window, makeWindow )
 -- ineffective because the window can not yet be displayed.
 --
 -- The value returned by 'createWindow' and 'createSubWindow' is a unique
--- identifier for the window, which can be used when calling 'setWindow'.
+-- identifier for the window, which can be used with 'currentWindow'.
 
 -- | Create a top-level window. The given name will be provided to the window
 -- system as the window\'s name. The intent is that the window system will label
@@ -131,12 +131,12 @@ foreign import CALLCONV unsafe "glutCreateSubWindow" glutCreateSubWindow ::
 --------------------------------------------------------------------------------
 
 -- | Contains the /current window\'s/ parent. If the /current window/ is a
--- top-level window, a pseudo window is returned, see 'isRealWindow'.
+-- top-level window, 'Nothing' is returned.
 
-parentWindow :: GettableStateVar Window
+parentWindow :: GettableStateVar (Maybe Window)
 parentWindow =
    makeGettableStateVar $
-      simpleGet makeWindow glut_WINDOW_PARENT
+      getWindow (simpleGet makeWindow glut_WINDOW_PARENT)
 
 --------------------------------------------------------------------------------
 
@@ -154,8 +154,8 @@ numSubWindows =
 -- logical colormap (if the window is color index), and overlay and related
 -- state (if an overlay has been established). Any subwindows of the destroyed
 -- window are also destroyed by 'destroyWindow'. If the specified window was the
--- /current window/, the /current window/ becomes invalid ('getWindow' will
--- return 'Nothing').
+-- /current window/, the /current window/ becomes invalid ('currentWindow' will
+-- contain 'Nothing').
 
 foreign import CALLCONV unsafe "glutDestroyWindow" destroyWindow ::
    Window -> IO ()
@@ -163,23 +163,23 @@ foreign import CALLCONV unsafe "glutDestroyWindow" destroyWindow ::
 --------------------------------------------------------------------------------
 
 -- | Controls the /current window/. It does /not/ affect the /layer in use/ for
--- the window; this is done using 'Graphics.UI.GLUT.Overlay.layerInUse'. If no
--- windows exist or the previously /current window/ was destroyed, a pseudo
--- window is returned, see 'isRealWindow'.
+-- the window; this is done using 'Graphics.UI.GLUT.Overlay.layerInUse'.
+-- Contains 'Nothing' if no windows exist or the previously /current window/ was
+-- destroyed. Setting the /current window/ to 'Nothing' is a no-op.
 
-currentWindow :: StateVar Window
-currentWindow = makeStateVar glutGetWindow glutSetWindow
+currentWindow :: StateVar (Maybe Window)
+currentWindow =
+   makeStateVar (getWindow glutGetWindow) (maybe (return ()) glutSetWindow)
+
+getWindow :: IO Window -> IO (Maybe Window)
+getWindow act = do
+   win <- act
+   return $ if win == makeWindow 0 then Nothing else Just win
 
 foreign import CALLCONV unsafe "glutSetWindow" glutSetWindow :: Window -> IO ()
 
 foreign import CALLCONV unsafe "glutGetWindow" glutGetWindow :: IO Window
 
--- | Returns 'True' if the given window identifier refers to a real window, not
--- a pseudo one.
-
-isRealWindow :: Window -> Bool
-isRealWindow = (/= makeWindow 0)
-
 --------------------------------------------------------------------------------
 
 -- | Mark the normal plane of given window (or the /current window/, if none
@@ -205,9 +205,10 @@ foreign import CALLCONV unsafe "glutPostRedisplay" glutPostRedisplay :: IO ()
 -- | Mark the normal plane of the given window as needing to be redisplayed,
 -- otherwise the same as 'postRedisplay'.
 --
--- The advantage of this routine is that it saves the cost of a 'setWindow' call
--- (entailing an expensive OpenGL context switch), which is particularly useful
--- when multiple windows need redisplays posted at the same time. 
+-- The advantage of this routine is that it saves the cost of using
+-- 'currentWindow' (entailing an expensive OpenGL context switch), which is
+-- particularly useful when multiple windows need redisplays posted at the same
+-- time. 
 
 foreign import CALLCONV unsafe "glutPostWindowRedisplay"
    glutPostWindowRedisplay :: Window -> IO ()
@@ -251,7 +252,7 @@ foreign import CALLCONV unsafe "glutSwapBuffers" swapBuffers :: IO ()
 -- In the case of top-level windows, setting 'windowPosition' is considered only
 -- a request for positioning the window. The window system is free to apply its
 -- own policies to top-level window placement. The intent is that top-level
--- windows should be repositioned according 'windowPosition'\'s value.
+-- windows should be repositioned according to the value of 'windowPosition'.
 
 windowPosition :: StateVar Position
 windowPosition = makeStateVar getWindowPosition setWindowPosition
@@ -277,7 +278,7 @@ getWindowPosition = do
 -- In the case of top-level windows, setting 'windowSize' is considered only a
 -- request for sizing the window. The window system is free to apply its own
 -- policies to top-level window sizing. The intent is that top-level windows
--- should be reshaped according 'windowSize'\'s value. Whether a reshape
+-- should be reshaped according to the value of 'windowSize'. Whether a reshape
 -- actually takes effect and, if so, the reshaped dimensions are reported to the
 -- program by a reshape callback.
 
@@ -430,57 +431,57 @@ data Cursor
    deriving ( Eq, Ord, Show )
 
 marshalCursor :: Cursor -> CInt
-marshalCursor c = case c of
-   RightArrow        -> glut_CURSOR_RIGHT_ARROW
-   LeftArrow         -> glut_CURSOR_LEFT_ARROW
-   Info              -> glut_CURSOR_INFO
-   Destroy           -> glut_CURSOR_DESTROY
-   Help              -> glut_CURSOR_HELP
-   Cycle             -> glut_CURSOR_CYCLE
-   Spray             -> glut_CURSOR_SPRAY
-   Wait              -> glut_CURSOR_WAIT
-   Text              -> glut_CURSOR_TEXT
-   Crosshair         -> glut_CURSOR_CROSSHAIR
-   UpDown            -> glut_CURSOR_UP_DOWN
-   LeftRight         -> glut_CURSOR_LEFT_RIGHT
-   TopSide           -> glut_CURSOR_TOP_SIDE
-   BottomSide        -> glut_CURSOR_BOTTOM_SIDE
-   LeftSide          -> glut_CURSOR_LEFT_SIDE
-   RightSide         -> glut_CURSOR_RIGHT_SIDE
-   TopLeftCorner     -> glut_CURSOR_TOP_LEFT_CORNER
-   TopRightCorner    -> glut_CURSOR_TOP_RIGHT_CORNER
+marshalCursor x = case x of
+   RightArrow -> glut_CURSOR_RIGHT_ARROW
+   LeftArrow -> glut_CURSOR_LEFT_ARROW
+   Info -> glut_CURSOR_INFO
+   Destroy -> glut_CURSOR_DESTROY
+   Help -> glut_CURSOR_HELP
+   Cycle -> glut_CURSOR_CYCLE
+   Spray -> glut_CURSOR_SPRAY
+   Wait -> glut_CURSOR_WAIT
+   Text -> glut_CURSOR_TEXT
+   Crosshair -> glut_CURSOR_CROSSHAIR
+   UpDown -> glut_CURSOR_UP_DOWN
+   LeftRight -> glut_CURSOR_LEFT_RIGHT
+   TopSide -> glut_CURSOR_TOP_SIDE
+   BottomSide -> glut_CURSOR_BOTTOM_SIDE
+   LeftSide -> glut_CURSOR_LEFT_SIDE
+   RightSide -> glut_CURSOR_RIGHT_SIDE
+   TopLeftCorner -> glut_CURSOR_TOP_LEFT_CORNER
+   TopRightCorner -> glut_CURSOR_TOP_RIGHT_CORNER
    BottomRightCorner -> glut_CURSOR_BOTTOM_RIGHT_CORNER
-   BottomLeftCorner  -> glut_CURSOR_BOTTOM_LEFT_CORNER
-   Inherit           -> glut_CURSOR_INHERIT
-   None              -> glut_CURSOR_NONE
-   FullCrosshair     -> glut_CURSOR_FULL_CROSSHAIR
+   BottomLeftCorner -> glut_CURSOR_BOTTOM_LEFT_CORNER
+   Inherit -> glut_CURSOR_INHERIT
+   None -> glut_CURSOR_NONE
+   FullCrosshair -> glut_CURSOR_FULL_CROSSHAIR
 
 unmarshalCursor :: CInt -> Cursor
-unmarshalCursor c
-   | c == glut_CURSOR_RIGHT_ARROW         = RightArrow
-   | c == glut_CURSOR_LEFT_ARROW          = LeftArrow
-   | c == glut_CURSOR_INFO                = Info
-   | c == glut_CURSOR_DESTROY             = Destroy
-   | c == glut_CURSOR_HELP                = Help
-   | c == glut_CURSOR_CYCLE               = Cycle
-   | c == glut_CURSOR_SPRAY               = Spray
-   | c == glut_CURSOR_WAIT                = Wait
-   | c == glut_CURSOR_TEXT                = Text
-   | c == glut_CURSOR_CROSSHAIR           = Crosshair
-   | c == glut_CURSOR_UP_DOWN             = UpDown
-   | c == glut_CURSOR_LEFT_RIGHT          = LeftRight
-   | c == glut_CURSOR_TOP_SIDE            = TopSide
-   | c == glut_CURSOR_BOTTOM_SIDE         = BottomSide
-   | c == glut_CURSOR_LEFT_SIDE           = LeftSide
-   | c == glut_CURSOR_RIGHT_SIDE          = RightSide
-   | c == glut_CURSOR_TOP_LEFT_CORNER     = TopLeftCorner
-   | c == glut_CURSOR_TOP_RIGHT_CORNER    = TopRightCorner
-   | c == glut_CURSOR_BOTTOM_RIGHT_CORNER = BottomRightCorner
-   | c == glut_CURSOR_BOTTOM_LEFT_CORNER  = BottomLeftCorner
-   | c == glut_CURSOR_INHERIT             = Inherit
-   | c == glut_CURSOR_NONE                = None
-   | c == glut_CURSOR_FULL_CROSSHAIR      = FullCrosshair
-   | otherwise = error "unmarshalCursor"
+unmarshalCursor x
+   | x == glut_CURSOR_RIGHT_ARROW = RightArrow
+   | x == glut_CURSOR_LEFT_ARROW = LeftArrow
+   | x == glut_CURSOR_INFO = Info
+   | x == glut_CURSOR_DESTROY = Destroy
+   | x == glut_CURSOR_HELP = Help
+   | x == glut_CURSOR_CYCLE = Cycle
+   | x == glut_CURSOR_SPRAY = Spray
+   | x == glut_CURSOR_WAIT = Wait
+   | x == glut_CURSOR_TEXT = Text
+   | x == glut_CURSOR_CROSSHAIR = Crosshair
+   | x == glut_CURSOR_UP_DOWN = UpDown
+   | x == glut_CURSOR_LEFT_RIGHT = LeftRight
+   | x == glut_CURSOR_TOP_SIDE = TopSide
+   | x == glut_CURSOR_BOTTOM_SIDE = BottomSide
+   | x == glut_CURSOR_LEFT_SIDE = LeftSide
+   | x == glut_CURSOR_RIGHT_SIDE = RightSide
+   | x == glut_CURSOR_TOP_LEFT_CORNER = TopLeftCorner
+   | x == glut_CURSOR_TOP_RIGHT_CORNER = TopRightCorner
+   | x == glut_CURSOR_BOTTOM_RIGHT_CORNER = BottomRightCorner
+   | x == glut_CURSOR_BOTTOM_LEFT_CORNER = BottomLeftCorner
+   | x == glut_CURSOR_INHERIT = Inherit
+   | x == glut_CURSOR_NONE = None
+   | x == glut_CURSOR_FULL_CROSSHAIR = FullCrosshair
+   | otherwise = error ("unmarshalCursor: illegal value " ++ show x)
 
 --------------------------------------------------------------------------------
 
