@@ -35,26 +35,25 @@ module Graphics.Rendering.OpenGL.GLU.Tessellation (
    Primitive(..), SimplePolygon(..), tessellate
 ) where
 
-import Control.Monad ( foldM, liftM )
-import Data.Either ( Either )
+import Control.Monad ( foldM, liftM, unless )
 import Data.IORef ( newIORef, readIORef, writeIORef, modifyIORef )
 import Foreign.Marshal.Alloc ( allocaBytes )
 import Foreign.Marshal.Array ( peekArray, pokeArray )
 import Foreign.Marshal.Pool ( Pool, withPool, pooledNew )
 import Foreign.Ptr ( Ptr, nullPtr, plusPtr, castPtr, FunPtr, freeHaskellFunPtr )
 import Foreign.Storable ( Storable(..) )
-import Graphics.Rendering.OpenGL.GL.BasicTypes ( GLclampf, GLdouble, GLenum )
+import Graphics.Rendering.OpenGL.GL.BasicTypes (
+   GLboolean, GLclampf, GLdouble, GLenum )
 import Graphics.Rendering.OpenGL.GL.EdgeFlag ( unmarshalEdgeFlag )
-import Graphics.Rendering.OpenGL.GL.Exception ( finally )
-import Graphics.Rendering.OpenGL.GL.GLboolean ( GLboolean, marshalGLboolean )
+import Graphics.Rendering.OpenGL.GL.Exception ( bracket )
+import Graphics.Rendering.OpenGL.GL.GLboolean ( marshalGLboolean )
 import Graphics.Rendering.OpenGL.GL.PrimitiveMode ( unmarshalPrimitiveMode )
 import Graphics.Rendering.OpenGL.GL.BeginEnd (
    PrimitiveMode, EdgeFlag(BeginsInteriorEdge) )
 import Graphics.Rendering.OpenGL.GL.VertexSpec (
    Vertex3(..), Normal3(..) )
-import Graphics.Rendering.OpenGL.GLU.ErrorsInternal ( makeError )
-import Graphics.Rendering.OpenGL.GLU.Errors (
-   Error(Error), ErrorCategory(OutOfMemory) )
+import Graphics.Rendering.OpenGL.GLU.ErrorsInternal (
+   recordErrorCode, recordOutOfMemory )
 
 --------------------------------------------------------------------------------
 
@@ -279,7 +278,7 @@ type Tessellator p v
   -> Normal3 GLdouble
   -> Combiner v
   -> ComplexPolygon v
-  -> IO (Either Error (p v))
+  -> IO (p v)
 
 --------------------------------------------------------------------------------
 
@@ -320,7 +319,7 @@ extractContours windingRule tolerance normal combiner complexPoly = do
 
        getContours = liftM (PolygonContours . reverse) (readIORef contours)
 
-   withTessellatorObj $ \tessObj -> do
+   withTessellatorObj (PolygonContours [])$ \tessObj -> do
       setTessellatorProperties tessObj windingRule tolerance normal True
       withVertexCallback tessObj addVertex $
          withEndCallback tessObj finishContour $
@@ -373,7 +372,7 @@ triangulate windingRule tolerance normal combiner complexPoly = do
           vs <- readIORef vertices
           return $ Triangulation (collectTriangles (reverse vs))
 
-   withTessellatorObj $ \tessObj -> do
+   withTessellatorObj (Triangulation []) $ \tessObj -> do
       setTessellatorProperties tessObj windingRule tolerance normal False
       withEdgeFlagCallback tessObj registerEdgeFlag $
          withVertexCallback tessObj addVertex $
@@ -425,7 +424,7 @@ tessellate windingRule tolerance normal combiner complexPoly = do
 
        getSimplePolygon = liftM (SimplePolygon . reverse) (readIORef primitives)
 
-   withTessellatorObj $ \tessObj -> do
+   withTessellatorObj (SimplePolygon []) $ \tessObj -> do
       setTessellatorProperties tessObj windingRule tolerance normal False
       withBeginCallback tessObj setPrimitiveMode $
          withVertexCallback tessObj addVertex $
@@ -438,19 +437,25 @@ tessellate windingRule tolerance normal combiner complexPoly = do
 --------------------------------------------------------------------------------
 -- chapter 5.1: The Tessellation Object
 
--- 'Char' is a fake here, any marshalable type would do
-newtype TessellatorObj = TessellatorObj (Ptr Char)
-   deriving ( Eq )
+-- an opaque pointer to a tessellator object
+newtype TessellatorObj = TessellatorObj (Ptr TessellatorObj)
 
-withTessellatorObj ::
-   (TessellatorObj -> IO (Either Error a)) -> IO (Either Error a)
-withTessellatorObj action = do
-   tessObj <- gluNewTess
-   if tessObj == TessellatorObj nullPtr
-      then return $ Left (Error OutOfMemory "out of memory")
-      else action tessObj `finally` gluDeleteTess tessObj
+isNullTesselatorObj :: TessellatorObj -> Bool
+isNullTesselatorObj (TessellatorObj ptr) = ptr == nullPtr
 
-foreign import CALLCONV unsafe "gluNewTess" gluNewTess :: IO (TessellatorObj)
+withTessellatorObj :: a -> (TessellatorObj -> IO a) -> IO a
+withTessellatorObj failureValue action =
+   bracket gluNewTess safeDeleteTess
+           (\tessObj -> if isNullTesselatorObj tessObj
+                           then do recordOutOfMemory
+                                   return failureValue
+                           else action tessObj)
+
+foreign import CALLCONV unsafe "gluNewTess" gluNewTess :: IO TessellatorObj
+
+safeDeleteTess :: TessellatorObj -> IO ()
+safeDeleteTess tessObj =
+   unless (isNullTesselatorObj tessObj) $ gluDeleteTess tessObj
 
 foreign import CALLCONV unsafe "gluDeleteTess" gluDeleteTess ::
    TessellatorObj -> IO ()
@@ -524,12 +529,13 @@ type BeginCallback  = PrimitiveMode -> IO ()
 type BeginCallback' = GLenum -> IO ()
 
 withBeginCallback :: TessellatorObj -> BeginCallback -> IO a -> IO a
-withBeginCallback tessObj beginCallback action = do
-   callbackPtr <- makeBeginCallback (beginCallback . unmarshalPrimitiveMode)
-   setBeginCallback tessObj (marshalTessCallback TessBegin) callbackPtr
-   action `finally` freeHaskellFunPtr callbackPtr
+withBeginCallback tessObj beginCallback action =
+   bracket (makeBeginCallback (beginCallback . unmarshalPrimitiveMode))
+           freeHaskellFunPtr $ \callbackPtr -> do
+      setBeginCallback tessObj (marshalTessCallback TessBegin) callbackPtr
+      action
 
-foreign import ccall "wrapper" makeBeginCallback ::
+foreign import CALLCONV "wrapper" makeBeginCallback ::
    BeginCallback' -> IO (FunPtr BeginCallback')
 
 foreign import CALLCONV unsafe "gluTessCallback" setBeginCallback ::
@@ -543,12 +549,13 @@ type EdgeFlagCallback  = EdgeFlag -> IO ()
 type EdgeFlagCallback' = GLboolean -> IO ()
 
 withEdgeFlagCallback :: TessellatorObj -> EdgeFlagCallback -> IO a -> IO a
-withEdgeFlagCallback tessObj edgeFlagCallback action = do
-   callbackPtr <- makeEdgeFlagCallback (edgeFlagCallback . unmarshalEdgeFlag)
-   setEdgeFlagCallback tessObj (marshalTessCallback TessEdgeFlag) callbackPtr
-   action `finally` freeHaskellFunPtr callbackPtr
+withEdgeFlagCallback tessObj edgeFlagCallback action =
+   bracket (makeEdgeFlagCallback (edgeFlagCallback . unmarshalEdgeFlag))
+           freeHaskellFunPtr $ \callbackPtr -> do
+      setEdgeFlagCallback tessObj (marshalTessCallback TessEdgeFlag) callbackPtr
+      action
 
-foreign import ccall "wrapper" makeEdgeFlagCallback ::
+foreign import CALLCONV "wrapper" makeEdgeFlagCallback ::
    EdgeFlagCallback' -> IO (FunPtr EdgeFlagCallback')
 
 foreign import CALLCONV unsafe "gluTessCallback" setEdgeFlagCallback ::
@@ -563,12 +570,13 @@ type VertexCallback' v = Ptr (AnnotatedVertex v) -> IO ()
 
 withVertexCallback ::
    Storable v => TessellatorObj -> VertexCallback v -> IO a -> IO a
-withVertexCallback tessObj vertexCallback action = do
-   callbackPtr <- makeVertexCallback (\p -> peek p >>= vertexCallback)
-   setVertexCallback tessObj (marshalTessCallback TessVertex) callbackPtr
-   action `finally` freeHaskellFunPtr callbackPtr
+withVertexCallback tessObj vertexCallback action =
+   bracket (makeVertexCallback (\p -> peek p >>= vertexCallback))
+           freeHaskellFunPtr $ \callbackPtr -> do
+      setVertexCallback tessObj (marshalTessCallback TessVertex) callbackPtr
+      action
 
-foreign import ccall "wrapper" makeVertexCallback ::
+foreign import CALLCONV "wrapper" makeVertexCallback ::
    VertexCallback' v -> IO (FunPtr (VertexCallback' v))
 
 foreign import CALLCONV unsafe "gluTessCallback" setVertexCallback ::
@@ -580,12 +588,12 @@ foreign import CALLCONV unsafe "gluTessCallback" setVertexCallback ::
 type EndCallback  = IO ()
 
 withEndCallback :: TessellatorObj -> EndCallback -> IO a -> IO a
-withEndCallback tessObj endCallback action = do
-   callbackPtr <- makeEndCallback endCallback
-   setEndCallback tessObj (marshalTessCallback TessEnd) callbackPtr
-   action `finally` freeHaskellFunPtr callbackPtr
+withEndCallback tessObj endCallback action =
+   bracket (makeEndCallback endCallback) freeHaskellFunPtr $ \callbackPtr -> do
+      setEndCallback tessObj (marshalTessCallback TessEnd) callbackPtr
+      action
 
-foreign import ccall "wrapper" makeEndCallback ::
+foreign import CALLCONV "wrapper" makeEndCallback ::
    EndCallback -> IO (FunPtr EndCallback)
 
 foreign import CALLCONV unsafe "gluTessCallback" setEndCallback ::
@@ -594,29 +602,23 @@ foreign import CALLCONV unsafe "gluTessCallback" setEndCallback ::
 --------------------------------------------------------------------------------
 -- chapter 5.3: Callbacks (error)
 
-type ErrorCallback  = Error -> IO ()
-
-type ErrorCallback' = GLenum -> IO ()
+type ErrorCallback = GLenum -> IO ()
 
 withErrorCallback :: TessellatorObj -> ErrorCallback -> IO a -> IO a
-withErrorCallback tessObj errorCallback action = do
-   callbackPtr <- makeErrorCallback (\e -> makeError e >>= errorCallback)
-   setErrorCallback tessObj (marshalTessCallback TessError) callbackPtr
-   action `finally` freeHaskellFunPtr callbackPtr
+withErrorCallback tessObj errorCallback action =
+   bracket (makeErrorCallback errorCallback)
+           freeHaskellFunPtr $ \callbackPtr -> do
+      setErrorCallback tessObj (marshalTessCallback TessError) callbackPtr
+      action
 
-foreign import ccall "wrapper" makeErrorCallback ::
-   ErrorCallback' -> IO (FunPtr ErrorCallback')
+foreign import CALLCONV "wrapper" makeErrorCallback ::
+   ErrorCallback -> IO (FunPtr ErrorCallback)
 
 foreign import CALLCONV unsafe "gluTessCallback" setErrorCallback ::
-   TessellatorObj -> GLenum -> FunPtr ErrorCallback' -> IO ()
+   TessellatorObj -> GLenum -> FunPtr ErrorCallback -> IO ()
 
-checkForError :: TessellatorObj -> IO a -> IO (Either Error a)
-checkForError tessObj action = do
-   maybeErrorRef <- newIORef Nothing
-   withErrorCallback tessObj (writeIORef maybeErrorRef . Just) $ do
-      res <- action
-      maybeError <- readIORef maybeErrorRef
-      return $ maybe (Right res) Left maybeError
+checkForError :: TessellatorObj -> IO a -> IO a
+checkForError tessObj = withErrorCallback tessObj recordErrorCode
 
 --------------------------------------------------------------------------------
 -- chapter 5.3: Callbacks (combine)
@@ -631,11 +633,11 @@ type CombineCallback v =
 withCombineCallback ::
    Storable v => TessellatorObj -> Combiner v -> IO a -> IO a
 withCombineCallback tessObj combiner action =
-   withPool $ \vertexPool -> do
-      let callback = combineProperties vertexPool combiner
-      callbackPtr <- makeCombineCallback callback
-      setCombineCallback tessObj (marshalTessCallback TessCombine) callbackPtr
-      action `finally` freeHaskellFunPtr callbackPtr
+   withPool $ \vertexPool ->
+      bracket (makeCombineCallback (combineProperties vertexPool combiner))
+              freeHaskellFunPtr $ \callbackPtr -> do
+         setCombineCallback tessObj (marshalTessCallback TessCombine) callbackPtr
+         action 
 
 combineProperties :: Storable v => Pool -> Combiner v -> CombineCallback v
 combineProperties pool combiner newVertexPtr propertyPtrs weights result = do
@@ -651,7 +653,7 @@ getProperty propertyPtrs n = do
    AnnotatedVertex _ v <- peek =<< peekElemOff propertyPtrs n
    return v
 
-foreign import ccall "wrapper" makeCombineCallback ::
+foreign import CALLCONV "wrapper" makeCombineCallback ::
    CombineCallback v -> IO (FunPtr (CombineCallback v))
 
 foreign import CALLCONV unsafe "gluTessCallback" setCombineCallback ::
@@ -675,7 +677,7 @@ setWindingRule tessObj =
 
 setBoundaryOnly :: TessellatorObj -> Bool -> IO ()
 setBoundaryOnly tessObj =
-   tessProperty tessObj TessBoundaryOnly . fromIntegral . marshalGLboolean
+   tessProperty tessObj TessBoundaryOnly . marshalGLboolean
 
 setTolerance :: TessellatorObj -> Tolerance -> IO ()
 setTolerance tessObj =
